/*
 *------------------------------------------------------------------
 * Flow data export record definitions.
 *
 * March 1997, Darren Kerr and Dave Rowell
 *
 * Copyright (c) 1997, 1998 by cisco Systems, Inc.
 * All rights reserved.
 *------------------------------------------------------------------
 */

#ifndef __IPFAST_FLOW_EXPORT_H__
#define __IPFAST_FLOW_EXPORT_H__

#ifndef MAX
#define MAX(a,b) ((a) >= (b) ? (a) : (b))
#endif

/* Index exported when there is no interface */
#define FLOW_INDEX_UNKNOWN 0x0000

/*
 * Defines for the flow switching engine type
 */
enum flow_engine_types {
    FLOW_ENGINE_RP = 0,
    FLOW_ENGINE_VIP
};

typedef struct {
    ushort version;         /* Current version */
    ushort count;           /* The number of records in PDU. */
    ulong  SysUptime;       /* Current time in millisecs since router booted */
    ulong  unix_secs;       /* Current seconds since 0000 UTC 1970 */
    ulong  unix_nsecs;      /* Residual nanoseconds since 0000 UTC 1970 */
} FlowStatHdrV1;

#define MAX_FLOWS_PER_V1_PAK   24

typedef struct {
    ipaddrtype srcaddr;    /* Source IP Address */
    ipaddrtype dstaddr;    /* Destination IP Address */
    ipaddrtype nexthop;    /* Next hop router's IP Address */
    ushort input;          /* Input interface index */
    ushort output;         /* Output interface index */
    
    ulong dPkts;           /* Packets sent in Duration */
    ulong dOctets;         /* Octets sent in Duration. */
    ulong First;           /* SysUptime at start of flow */
    ulong Last;            /* and of last packet of flow */
    
    ushort srcport;        /* TCP/UDP source port number or equivalent */
    ushort dstport;        /* TCP/UDP destination port number or equivalent */
    ushort pad;
    uchar  prot;           /* IP protocol, e.g., 6=TCP, 17=UDP, ... */
    uchar  tos;            /* IP Type-of-Service */
    
    uchar  flags;          /* Reason flow was discarded, etc...  */
    uchar  tcp_retx_cnt;   /* Number of mis-seq with delay > 1sec */
    uchar  tcp_retx_secs;  /* Cumulative seconds between mis-sequenced pkts */
    uchar  tcp_misseq_cnt; /* Number of mis-sequenced tcp pkts seen */
    ulong  reserved;
} IPFlowStatV1;

typedef struct {
    FlowStatHdrV1 header;
    IPFlowStatV1  records[0];
} IPStatMsgV1;

#define MAX_FLOWS_PER_V5_PAK   30

typedef struct {
    ushort version;         /* Current version */
    ushort count;           /* The number of records in PDU. */
    ulong  SysUptime;       /* Current time in millisecs since router booted */
    ulong  unix_secs;       /* Current seconds since 0000 UTC 1970 */
    ulong  unix_nsecs;      /* Residual nanoseconds since 0000 UTC 1970 */
    ulong  flow_sequence;   /* Seq counter of total flows seen */
    uchar  engine_type;     /* Type of flow switching engine */
    uchar  engine_id;       /* ID number of the flow switching engine */
    ushort reserved;
} FlowStatHdrV5;

typedef struct {
    ipaddrtype srcaddr;    /* Source IP Address */
    ipaddrtype dstaddr;    /* Destination IP Address */
    ipaddrtype nexthop;    /* Next hop router's IP Address */
    ushort input;          /* Input interface index */
    ushort output;         /* Output interface index */
    
    ulong dPkts;           /* Packets sent in Duration */
    ulong dOctets;         /* Octets sent in Duration. */
    ulong First;           /* SysUptime at start of flow */
    ulong Last;            /* and of last packet of flow */
    
    ushort srcport;        /* TCP/UDP source port number or equivalent */
    ushort dstport;        /* TCP/UDP destination port number or equivalent */
    uchar  rsvd;
    uchar  tcp_flags;      /* Cumulative OR of tcp flags */
    uchar  prot;           /* IP protocol, e.g., 6=TCP, 17=UDP, ... */
    uchar  tos;            /* IP Type-of-Service */
    ushort src_as;         /* originating AS of source address */
    ushort dst_as;         /* originating AS of destination address */
    uchar  src_mask;       /* source address prefix mask bits */
    uchar  dst_mask;       /* destination address prefix mask bits */
    ushort pad;
} IPFlowStatV5;

typedef struct {
    FlowStatHdrV5 header;
    IPFlowStatV5  records[0];
} IPStatMsgV5;

#define MAX_FLOWS_PER_V6_PAK   27

typedef struct {
    ushort version;         /* Current version */
    ushort count;           /* The number of records in PDU. */
    ulong  SysUptime;       /* Current time in millisecs since router booted */
    ulong  unix_secs;       /* Current seconds since 0000 UTC 1970 */
    ulong  unix_nsecs;      /* Residual nanoseconds since 0000 UTC 1970 */
    ulong  flow_sequence;   /* Seq counter of total flows seen */
    uchar  engine_type;     /* Type of flow switching engine */
    uchar  engine_id;       /* ID number of the flow switching engine */
    ushort reserved;
} FlowStatHdrV6;

typedef struct {
    ipaddrtype srcaddr;    /* Source IP Address */
    ipaddrtype dstaddr;    /* Destination IP Address */
    ipaddrtype nexthop;    /* Next hop router's IP Address */
    ushort input;          /* Input interface index */
    ushort output;         /* Output interface index */
    
    ulong dPkts;           /* Packets sent in Duration */
    ulong dOctets;         /* Octets sent in Duration. */
    ulong First;           /* SysUptime at start of flow */
    ulong Last;            /* and of last packet of flow */
    
    ushort srcport;        /* TCP/UDP source port number or equivalent */
    ushort dstport;        /* TCP/UDP destination port number or equivalent */
    uchar  rsvd;
    uchar  tcp_flags;      /* Cumulative OR of tcp flags */
    uchar  prot;           /* IP protocol, e.g., 6=TCP, 17=UDP, ... */
    uchar  tos;            /* IP Type-of-Service */
    ushort src_as;         /* originating AS of source address */
    ushort dst_as;         /* originating AS of destination address */
    uchar  src_mask;       /* source address prefix mask bits */
    uchar  dst_mask;       /* destination address prefix mask bits */
    uchar  in_encaps;      /* size in bytes of the input encapsulation */
    uchar  out_encaps;     /* size in bytes of the output encapsulation */
    ulong  peer_nexthop;   /* IP address of the nexthop within the peer (FIB)*/
} IPFlowStatV6;

typedef struct {
    FlowStatHdrV6 header;
    IPFlowStatV6  records[0];
} IPStatMsgV6;

typedef struct {
    ushort version;         /* Current version */
    ushort count;           /* The number of records in PDU. */
    ulong  SysUptime;       /* Current time in millisecs since router booted */
    ulong  unix_secs;       /* Current seconds since 0000 UTC 1970 */
    ulong  unix_nsecs;      /* Residual nanoseconds since 0000 UTC 1970 */
    ulong  flow_sequence;   /* Seq counter of total flows seen */
    uchar  engine_type;     /* Type of flow switching engine */
    uchar  engine_id;       /* ID number of the flow switching engine */
    uchar  aggregation;     /* Aggregation method being used */
    uchar  agg_version;     /* Version of the aggregation export */
    ulong  reserved;
} FlowStatHdrV8;

/*
 * Flow aggregation schemes
 */
enum {
    IPFLOW_NO_AGGREGATION,
    IPFLOW_AS_AGGREGATION,
    IPFLOW_PROTO_PORT_AGGREGATION,
    IPFLOW_SRC_PREFIX_AGGREGATION,
    IPFLOW_DST_PREFIX_AGGREGATION,
    IPFLOW_PREFIX_AGGREGATION,
    IPFLOW_MAX_AGGREGATION
};

/*
 * Flow stat record for AS aggregation
 */
typedef struct {
    ulong      flows;          /* Number of flows */
    ulong      dPkts;          /* Packets sent in Duration */
    ulong      dOctets;        /* Octets sent in Duration. */
    ulong      First;          /* SysUptime at start of flow */
    ulong      Last;           /* and of last packet of flow */
    ushort     src_as;         /* originating AS of source address */
    ushort     dst_as;         /* originating AS of destination address */
    ushort     input;          /* Input interface index */
    ushort     output;         /* Output interface index */
} IPFlowStatV8_AS;

#define V8_AS_VERSION 2          /* Current version of the AS record */
#define MAX_FLOWS_PER_V8_AS_PAK 51

/*
 * Flow stat record for Protocol Port aggregation
 */
typedef struct {
    ulong      flows;          /* Number of flows */
    ulong      dPkts;          /* Packets sent in Duration */
    ulong      dOctets;        /* Octets sent in Duration. */
    ulong      First;          /* SysUptime at start of flow */
    ulong      Last;           /* and of last packet of flow */
    uchar      prot;           /* IP protocol, e.g., 6=TCP, 17=UDP, ... */
    uchar      pad;
    ushort     reserved;
    ushort     srcport;        /* TCP/UDP source port number or equivalent */
    ushort     dstport;        /* TCP/UDP dest port number or equivalent */
} IPFlowStatV8_ProtoPort;

#define V8_PROTO_PORT_VERSION 2  /* Current version of the proto-port record */
#define MAX_FLOWS_PER_V8_PROTO_PORT_PAK 51

/*
 * Flow stat record for SourcePrefix aggregation
 */
typedef struct {
    ulong      flows;          /* Number of flows */
    ulong      dPkts;          /* Packets sent in Duration */
    ulong      dOctets;        /* Octets sent in Duration. */
    ulong      First;          /* SysUptime at start of flow */
    ulong      Last;           /* and of last packet of flow */
    ipaddrtype src_prefix;     /* Source prefix */
    uchar      src_mask;       /* source address prefix mask bits */
    uchar      pad;
    ushort     src_as;         /* originating AS of source address */
    ushort     input;          /* Input interface index */
    ushort     reserved;
} IPFlowStatV8_SrcPrefix;

#define V8_SRC_PREFIX_VERSION 2  /* Current version of the src prefix record */
#define MAX_FLOWS_PER_V8_SRC_PREFIX_PAK 44

/*
 * Flow stat record for DestinationPrefix aggregation
 */
typedef struct {
    ulong      flows;          /* Number of flows */
    ulong      dPkts;          /* Packets sent in Duration */
    ulong      dOctets;        /* Octets sent in Duration. */
    ulong      First;          /* SysUptime at start of flow */
    ulong      Last;           /* and of last packet of flow */
    ipaddrtype dst_prefix;     /* Destination prefix */
    uchar      dst_mask;       /* destination address prefix mask bits */
    uchar      pad;
    ushort     dst_as;         /* originating AS of destination address */
    ushort     output;         /* Output interface index */
    ushort     reserved;
} IPFlowStatV8_DstPrefix;

#define V8_DST_PREFIX_VERSION 2  /* Current version of the dst prefix record */
#define MAX_FLOWS_PER_V8_DST_PREFIX_PAK 44

/*
 * Flow stat record for Prefix aggregation
 */
typedef struct {
    ulong      flows;          /* Number of flows */
    ulong      dPkts;          /* Packets sent in Duration */
    ulong      dOctets;        /* Octets sent in Duration. */
    ulong      First;          /* SysUptime at start of flow */
    ulong      Last;           /* and of last packet of flow */
    ipaddrtype src_prefix;     /* Source prefix */
    ipaddrtype dst_prefix;     /* Destination prefix */
    uchar      dst_mask;       /* destination address prefix mask bits */
    uchar      src_mask;       /* source address prefix mask bits */
    ushort     reserved;
    ushort     src_as;         /* originating AS of source address */
    ushort     dst_as;         /* originating AS of destination address */
    ushort     input;          /* Input interface index */
    ushort     output;         /* Output interface index */
} IPFlowStatV8_Prefix;

#define V8_PREFIX_VERSION 2  /* Current version of the dst prefix record */
#define MAX_FLOWS_PER_V8_PREFIX_PAK 35

typedef struct {
    FlowStatHdrV8 header;
    union {
	IPFlowStatV8_AS        as[0];
	IPFlowStatV8_ProtoPort proto_port[0];
	IPFlowStatV8_SrcPrefix src_prefix[0];
	IPFlowStatV8_DstPrefix dst_prefix[0];
	IPFlowStatV8_Prefix    prefix[0];
    } records;
} IPStatMsgV8;

#define MAX_V1_FLOW_PAK_SIZE (sizeof(FlowStatHdrV1) + \
			      sizeof(IPFlowStatV1) * MAX_FLOWS_PER_V1_PAK)

#define MAX_V5_FLOW_PAK_SIZE (sizeof(FlowStatHdrV5) + \
			      sizeof(IPFlowStatV5) * MAX_FLOWS_PER_V5_PAK)

#define MAX_V6_FLOW_PAK_SIZE (sizeof(FlowStatHdrV6) + \
			      sizeof(IPFlowStatV6) * MAX_FLOWS_PER_V6_PAK)

#define MAX_V8_AS_FLOW_PAK_SIZE (sizeof(FlowStatHdrV8) +   \
				 sizeof(IPFlowStatV8_AS) * \
				 MAX_FLOWS_PER_V8_AS_PAK)
#define MAX_V8_PROTO_PORT_FLOW_PAK_SIZE (sizeof(FlowStatHdrV8) +          \
					 sizeof(IPFlowStatV8_ProtoPort) * \
					 MAX_FLOWS_PER_V8_PROTO_PORT_PAK)
#define MAX_V8_SRC_PREFIX_FLOW_PAK_SIZE (sizeof(FlowStatHdrV8) +          \
					 sizeof(IPFlowStatV8_SrcPrefix) * \
					 MAX_FLOWS_PER_V8_SRC_PREFIX_PAK)
#define MAX_V8_DST_PREFIX_FLOW_PAK_SIZE (sizeof(FlowStatHdrV8) +          \
					 sizeof(IPFlowStatV8_DstPrefix) * \
					 MAX_FLOWS_PER_V8_DST_PREFIX_PAK)
#define MAX_V8_PREFIX_FLOW_PAK_SIZE (sizeof(FlowStatHdrV8) +       \
				     sizeof(IPFlowStatV8_Prefix) * \
				     MAX_FLOWS_PER_V8_PREFIX_PAK)
#define MAX_V8_FLOW_PAK_SIZE MAX(MAX(MAX(MAX_V8_AS_FLOW_PAK_SIZE,           \
					 MAX_V8_PROTO_PORT_FLOW_PAK_SIZE),  \
				     MAX(MAX_V8_SRC_PREFIX_FLOW_PAK_SIZE,   \
					 MAX_V8_DST_PREFIX_FLOW_PAK_SIZE)), \
				 MAX_V8_PREFIX_FLOW_PAK_SIZE)
#define MAX_FLOW_PAK_SIZE MAX(MAX(MAX_V1_FLOW_PAK_SIZE,  \
				  MAX_V5_FLOW_PAK_SIZE), \
			      MAX(MAX_V6_FLOW_PAK_SIZE,  \
				  MAX_V8_FLOW_PAK_SIZE))

#endif __IPFAST_FLOW_EXPORT_H__
